--
-- Auction House Module
--

Crafty_AH = {}

-- Import
local AH = Crafty_AH
local Data = Crafty_Data
local QueueFrameList = Crafty_QueueFrameList
local TradeSkill = Crafty_TradeSkill
local UI = Crafty_UI
local Wait = Crafty_Wait
local CanSendAuctionQuery = CanSendAuctionQuery
local CreateFrame = CreateFrame
local GetItemInfo = GetItemInfo
local GetAuctionItemInfo = GetAuctionItemInfo
local GetAuctionItemLink = GetAuctionItemLink
local GetAuctionSellItemInfo = GetAuctionSellItemInfo
local GetNumAuctionItems = GetNumAuctionItems
local GetServerTime = GetServerTime
local IsAltKeyDown = IsAltKeyDown
local IsAuctionSortReversed = IsAuctionSortReversed
local MoneyInputFrame_SetCopper = MoneyInputFrame_SetCopper
local PriceDropDown = PriceDropDown
local QueryAuctionItems = QueryAuctionItems
local SortAuctionItems = SortAuctionItems
local UIDropDownMenu_SetSelectedValue = UIDropDownMenu_SetSelectedValue
local hooksecurefunc = hooksecurefunc
local math = math
local pairs = pairs
local string = string
local type = type
local wipe = wipe
local _G = _G

-- AuctionUI
LoadAddOn("Blizzard_AuctionUI")
local AuctionFrame = AuctionFrame
local AuctionFrameAuctions = AuctionFrameAuctions
local AuctionFrameBrowse = AuctionFrameBrowse
local AuctionFrame_SetSort = AuctionFrame_SetSort
local AuctionsStackSizeEntry = AuctionsStackSizeEntry
local BrowseBidPrice = BrowseBidPrice
local BrowseCurrentBidSort = BrowseCurrentBidSort
--local BrowseDropDown = BrowseDropDown
local BrowseDurationSort = BrowseDurationSort
local BrowseHighBidderSort = BrowseHighBidderSort
local BrowseLevelSort = BrowseLevelSort
local BrowseMinLevel = BrowseMinLevel
local BrowseMaxLevel = BrowseMaxLevel
local BrowseName = BrowseName
local BrowseNextPageButton = BrowseNextPageButton
local BrowsePrevPageButton = BrowsePrevPageButton
local BrowseQualitySort = BrowseQualitySort
local BrowseResetButton = BrowseResetButton
local BrowseSearchButton = BrowseSearchButton
local BuyoutPrice = BuyoutPrice
local ExactMatchCheckButton = ExactMatchCheckButton
local IsUsableCheckButton = IsUsableCheckButton
local ShowOnPlayerCheckButton = ShowOnPlayerCheckButton
local StartPrice = StartPrice

-- Make sure we don't polute the global environment
setfenv(1, {})

local SCAN_DELAY = 0.3		-- Delay between AH scan attempts
local BATCH_DELAY = 0.1		-- Delay between batches of item processing
local BATCH_LIMIT = 200		-- Max number of items to process in one batch
local PRICE_TYPE_STACK = 2	-- For Auctions "per stack"
local PROGRESS_CLOSE_DELAY = 3	-- How long to leave "scan done" up
local NOTICE_CLOSE_DELAY = 10   -- How long to leave no data notice up
local GETALL_CD = 900		-- Cooldown of getAll scan, in seconds
local SECS_PER_MIN = 60

local WARN_TEXT = Data.escapeColour.red .. "Warning:" .. Data.escapeColour.grey .. " If you have problems using the fast scan, ie disconnections or lockups, try the slow scan. This is enabled by holding the Alt key down whilst clicking the Scan button."

local SLOW_SCAN_TEXT = Data.escapeColour.yellow .. "Note:" .. Data.escapeColour.grey .. " Slow scan is limited to items from this character's professions that are being crafted for profit and reagents. This is so the scan can complete in a reasonable amount of time."

local NO_DATA_TEXT = Data.escapeColour.red .. "Notice:" .. Data.escapeColour.grey .. " You need to open your profession window(s) briefly before scanning the AH. This is so Crafty knows about all your recipes, this only needs to be done once for each character."

AH.scanning = {}		-- Data about current scan
AH.list = {}			-- List of items to scan
AH.seen = {}			-- List of items seen in a scan
AH.open = false
AH.stop = false

function AH:EnableButtons()
    self.scanButton:Enable()
    self.stopButton:Disable()

    -- Enable AH interface
    BrowseSearchButton:Enable()
    BrowseResetButton:Enable()
    BrowsePrevPageButton:Enable()
    BrowseNextPageButton:Enable()
    BrowseQualitySort:Enable()
    BrowseLevelSort:Enable()
    BrowseDurationSort:Enable()
    BrowseHighBidderSort:Enable()
    BrowseCurrentBidSort:Enable()
    ExactMatchCheckButton:Enable()
    BrowseName:Enable()
    BrowseMinLevel:Enable()
    BrowseMaxLevel:Enable()
    --BrowseDropDown:Enable()
    IsUsableCheckButton:Enable()
    ShowOnPlayerCheckButton:Enable()
end

function AH:DisableButtons()
    self.scanButton:Disable()
    self.stopButton:Enable()
    self.progressFrame:Show()

    -- Disable AH interface
    BrowseSearchButton:Disable()
    BrowseResetButton:Disable()
    BrowsePrevPageButton:Disable()
    BrowseNextPageButton:Disable()
    BrowseQualitySort:Disable()
    BrowseLevelSort:Disable()
    BrowseDurationSort:Disable()
    BrowseHighBidderSort:Disable()
    BrowseCurrentBidSort:Disable()
    ExactMatchCheckButton:Disable()
    BrowseName:Disable()
    BrowseMinLevel:Disable()
    BrowseMaxLevel:Disable()
    -- Can't disable a drop down
    --BrowseDropDown:Disable()
    IsUsableCheckButton:Disable()
    ShowOnPlayerCheckButton:Disable()
end 

function AH:OnOpen()
    self.open = true
    self.stop = false
end

function AH:OnClose()
    self:StopScan()
    self.open = false
end

function AH:IsScanRunning()
    return (type(self.scanning.scan) == "string")
end

function AH:StopScan()
    self.stop = true
    wipe(self.scanning)
    wipe(self.list)
    self:EnableButtons()
    self.progressFrame:Hide()
end

local function SetSort()
    -- Make sure we're sorting by unit price and the direction is right
    AuctionFrame_SetSort("list", "unitprice", false)
end

function AH:OnLoad()
    -- Move bid price across to the right
    BrowseBidPrice:ClearAllPoints()
    BrowseBidPrice:SetPoint("TOPRIGHT", "BrowseBidButton", "TOPLEFT")

    -- Browse Tab
    UI:CreateText("Crafty_ScanText", AuctionFrameBrowse, "GameFontNormalSmall",
	"Crafty:", "TOPLEFT", "AuctionFrameMoneyFrame", "TOPRIGHT", 4, -2)
    self.scanButton = UI:CreateButton("Crafty_ScanButton", 
	AuctionFrameBrowse, "Scan", 50, 22, "TOPLEFT", "Crafty_ScanText", 
	"TOPRIGHT", 0, 6, self.ScanAll, self)
    self.stopButton = UI:CreateButton("Crafty_StopButton", AuctionFrameBrowse, 
	"Stop", 50, 22, "TOPLEFT", "Crafty_ScanButton", "TOPRIGHT", 0, 0, 
	self.StopScan, self)
    self.stopButton:Disable()
    UI:CreateButton("Crafty_ShopButton", AuctionFrameBrowse, "Shop", 50, 22, 
	"TOPLEFT", "Crafty_StopButton", "TOPRIGHT", 0, 0, 
	QueueFrameList.OpenShop, QueueFrameList)

    -- Auctions Tab
    UI:CreateText("Crafty_UndercutText", AuctionFrameAuctions, 
	"GameFontNormalSmall", "Crafty:", "TOPLEFT", "AuctionFrameMoneyFrame", 
	"TOPRIGHT", 4, -2)
    UI:CreateButton("Crafty_UndercutButton", AuctionFrameAuctions, "Undercut", 
	80, 22, "TOPLEFT", "Crafty_UndercutText", "TOPRIGHT", 0, 6, 
	self.UndercutButton, self)
    self.undercutText = UI:CreateText("Crafty_AHUndercutText", 
	AuctionFrameAuctions, "GameFontNormalSmall", "", "TOPLEFT", 
	"Crafty_UndercutButton", "TOPRIGHT", 2, -6)

    -- Scan progress
    self.progressFrame = CreateFrame("Frame", "Crafty_AHProgressFrame", 
	AuctionFrameBrowse)
    self.progressFrame:SetFrameStrata("DIALOG")
    self.progressFrame:SetWidth(630)
    self.progressFrame:SetHeight(304)
    self.progressFrame:SetPoint("TOPLEFT", 190, -105)
    self.progressFrame:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	tile = true, tileSize = 16, edgeSize = 16,
    })
    self.progressFrame:SetBackdropColor(0, 0, 0, 0.99)
    self.progressFrame.Text = UI:CreateText("Crafty_AHProgressText", 
	self.progressFrame, "GameFontNormal", "", "CENTER", 
	self.progressFrame, "CENTER", 0, 0)
    self.progressFrame.Warn = UI:CreateText("Crafty_AHProgressWarn", 
	self.progressFrame, "GameFontNormal", "", "TOP", 
	"Crafty_AHProgressText", "BOTTOM", 0, -32, nil, 400)

    SetSort()
    self:EnableButtons()
    self.progressFrame:Hide()
end

-- Do a search for one item
function AH:SearchItem(item, exact)
    if self.stop or not self.open then return end

    -- Supports item to be either an itemID or itemName
    if type(item) == "number" then
	local itemName = GetItemInfo(item)
	if itemName then
	    item = itemName
	else
	    -- Not seen the item before so need to retry
	    --Data:Debug("AH:SearchItem", "Retrying GetItemInfo for", item)
	    Wait:Start(SCAN_DELAY, self.SearchItem, self, item, exact)
	    return
	end
    elseif type(item) == "string" then
	-- NOOP
    else
	return
    end

    if CanSendAuctionQuery() then
	-- Reset search
	BrowseResetButton:Click()
	if (exact and not ExactMatchCheckButton:GetChecked()) or 
	    (not exact and ExactMatchCheckButton:GetChecked()) then
	    ExactMatchCheckButton:Click()
	end
	-- Fill out the AH frame, rather than do a query, the result is 
	-- much nicer for the user!
	BrowseName:SetText(item)
	BrowseSearchButton:Click()
    else
	Wait:Start(SCAN_DELAY, self.SearchItem, self, item, exact)
	return
    end
end

function AH:HideProgress()
    -- If a scan has been started then cancel hiding the progress
    if self.scanning.scan then
	-- NOOP
    else
	self.progressFrame:Hide()
    end
end

-- Do a fast or slow scan of whole AH
function AH:ScanAll()
    if not self.open then return end

    if TradeSkill:HasTradeSkillData() then
	self.stop = false
	if IsAltKeyDown() then
	    self.progressFrame.Warn:SetText(SLOW_SCAN_TEXT)
	    self:SlowScan()
	else
	    self.progressFrame.Warn:SetText(WARN_TEXT)
	    self:GetAll()
	end
    else
	self.progressFrame.Warn:SetText(NO_DATA_TEXT)
	self.progressFrame.Text:SetText("")
	self.progressFrame:Show()
	Wait:Start(NOTICE_CLOSE_DELAY, self.HideProgress, self)
    end
end

-- Fast scan
function AH:GetAll()
    if self.stop or not self.open then return end

    local canQuery, canQueryAll = CanSendAuctionQuery()
    if canQueryAll then
	if canQuery then
	    wipe(self.seen)
	    self:DisableButtons()
	    self.lastGetAll = GetServerTime()
	    self.progressFrame.Text:SetText(Data.escapeColour.green .. 
		"Starting fast scan")
	    
	    -- "name", minLevel, maxLevel, page, isUsable, qualityIndex, getAll,
	    -- exactMatch, filterData
	    QueryAuctionItems(nil, nil, nil, nil, nil, nil, true, nil, nil)
	    self.scanning.scan = "getAll"
	    self.scanning.query = true
	else
	    Wait:Start(SCAN_DELAY, self.GetAll, self)
	    return
	end
    else
	local text = Data.escapeColour.red .. "Fast scan on cooldown"
	local time
	if self.lastGetAll then
	    time = GETALL_CD - (GetServerTime() - self.lastGetAll)
	    if time > 0 then
		local mins = math.ceil(time / SECS_PER_MIN)
		text = text .. " for about " .. mins .. " minutes"
	    end
	end
	self.progressFrame.Text:SetText(text)
	self.progressFrame:Show()
	Wait:Start(PROGRESS_CLOSE_DELAY, self.HideProgress, self)
	return
    end
end

-- Remove the "of the" part of an item name
local function GetAuctionName(itemName)
    return string.gsub(itemName, "of the .*", "")
end

-- Get undercut price
function AH:Undercut()
    if self.stop or not self.open then return end

    -- Make sure price is per stack
    if AuctionFrameAuctions.priceType ~= PRICE_TYPE_STACK then
	AuctionFrameAuctions.priceType = PRICE_TYPE_STACK
	UIDropDownMenu_SetSelectedValue(PriceDropDown, PRICE_TYPE_STACK)
    end

    local itemName, _, _, _, _, _, _, _, num = GetAuctionSellItemInfo()
    if itemName then
	itemName = GetAuctionName(itemName)
	self.list = { itemName }
	self:ScanList(function(item, _, buyoutEach)

	    if buyoutEach then
		if item == itemName then
		    local stackSize = AuctionsStackSizeEntry:GetNumber()
		    local buyout = buyoutEach * 
		    (num > stackSize and stackSize or num) - 1
		    MoneyInputFrame_SetCopper(BuyoutPrice, buyout)
		    MoneyInputFrame_SetCopper(StartPrice, buyout)
		    self.undercutText:SetText("Filled in undercut price")
		    Wait:Start(PROGRESS_CLOSE_DELAY, self.undercutText.SetText,
			self.undercutText, "")
		end
	    else
		self.undercutText:SetText("No auctions to undercut")
		Wait:Start(PROGRESS_CLOSE_DELAY, self.undercutText.SetText,
		    self.undercutText, "")
	    end
	end)
    else
	if self.retry then
	    -- NOOP
	else
	    --Data:Debug("AH:Undercut", "Retrying GetAuctionSellItemInfo")
	    Wait:Start(SCAN_DELAY, self.Undercut, self)
	    self.retry = true
	    return
	end
    end
end

-- Undercut button
function AH:UndercutButton()
    self.stop = false
    self.retry = false
    self.undercutText:SetText("Getting undercut price")
    self:Undercut()
end

-- Query one item from a list
function AH:QueryItem(item)
    if self.stop or not self.open then return end

    -- Supports item to be either an itemID or itemName
    if type(item) == "number" then
	local itemName = GetItemInfo(item)
	if itemName then
	    item = itemName
	else
	    -- Not seen the item before so need to retry
	    --Data:Debug("AH:QueryItem", "Retrying GetItemInfo for", item)
	    Wait:Start(SCAN_DELAY, self.QueryItem, self, item)
	    return
	end
    elseif type(item) == "string" then
	-- NOOP
    else
	return
    end

    if CanSendAuctionQuery() then
	-- "name", minLevel, maxLevel, page, isUsable, qualityIndex, getAll, 
	-- exactMatch, filterData
	QueryAuctionItems(item, nil, nil, self.scanning.page, nil, nil, false, 
	    false, nil)
	self.scanning.scan = "list"
	self.scanning.query = true
	self.scanning.retry = false
    else
	Wait:Start(SCAN_DELAY, self.QueryItem, self, item)
	return
    end
end

-- Scan items in AH.list and callback with the AH data
function AH:ScanList(callback)
    if self.stop or not self.open then return end

    -- Order doesn't matter so go from the end for efficiency
    item = self.list[#self.list]
    if item then 
	SetSort()
	self.scanning.callback = callback
	self.scanning.item = item
	self.scanning.page = 0
	self:QueryItem(item)
    end
end

-- Get the prices for the ScanList search and start the next query
function AH:GetListPrices(batch, total)
    if self.stop or not self.open then return end

    local found = false

    for i = 1, batch do
	-- name, texture, count, quality, canUse, level, levelColHeader, minBid,
	-- minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, 
	-- owner, ownerFullName, saleStatus, itemId, hasAllInfo
	local itemName, _, count, _, _, _, _, _, _, buyout, _, _, _, seller,
	    _, _, itemID = GetAuctionItemInfo("list", i)
	    
	if itemID and itemName then
	    itemName = GetAuctionName(itemName)
	    if itemID == self.scanning.item or itemName == self.scanning.item 
		then
		if count and count > 0 and buyout and buyout > 0 then
		    if seller then
			local buyoutEach = math.floor(buyout / count)
			-- If buyout is so low it could be rounded down to 0
			if buyoutEach == 0 then buyoutEach = 1 end
			local item = _G.Crafty_Items[itemID]
			if item and item.minBuyout then
			    _G.Crafty_Items[itemID].minBuyout = buyoutEach
			end
			self.scanning.callback(self.scanning.item, seller, 
			    buyoutEach)
			found = true
			break
		    else
			-- Assume we'll get the info on next update
			if self.scanning.retry then
			    -- NOOP
			else
			    self.scanning.query = true
			    self.scanning.retry = true
			    --[[
			    Data:Debug("AH:GetListPrices", 
				"Retrying GetAuctionItemInfo")
			    --]]
			    return
			end
		    end
		end
	    end
	else
	    -- Assume we'll get the info on next update
	    if self.scanning.retry then
		-- NOOP
	    else
		self.scanning.query = true
		self.scanning.retry = true
		--Data:Debug("AH:GetListPrices", "Retrying GetAuctionItemInfo")
		return
	    end
	end
    end

    local numPages = 0
    if batch > 0 then
	numPages = math.ceil(total / batch)
    end
    self.scanning.page = self.scanning.page + 1

    if not found and self.scanning.page > numPages - 1 then
	local _, itemLink = GetItemInfo(self.scanning.item)
	local itemID = Data:GetItemInfoFromLink(itemLink)
	local item = _G.Crafty_Items[itemID]
	if item and item.minBuyout then
	    _G.Crafty_Items[itemID].minBuyout = -1
	end
	self.scanning.callback(self.scanning.item, nil, nil)
    end

    if found or self.scanning.page > numPages - 1 then
	-- Next item
	self.list[#self.list] = nil
	if #self.list > 0 then
	    local item = self.list[#self.list]
	    self.scanning.item = item
	    self.scanning.page = 0
	    self:QueryItem(item)
	else
	    wipe(self.scanning)
	    --Data:Debug("AH:GetListPrices", "Scanning done")
	    return
	end
    else
	-- Next page
	self:QueryItem(self.scanning.item)
    end
end

-- Do a slow scan, which is really just a ScanList of a big list
function AH:SlowScan()
    if self.stop or not self.open then return end

    local total = 0
    local function SetProgressText(num)
	self.progressFrame.Text:SetText("Slow scanning " .. num .. 
	    " of " .. total ..  " items")
    end

    wipe(self.list)
    local items = _G.Crafty_Items
    for itemID in pairs(items) do
	local item = items[itemID]
	local hasTradeSkill = false
	if item and item.tradeSkillID then
	    -- Limit scan to professions this char has
	    hasTradeSkill = TradeSkill:HasTradeSkill(item.tradeSkillID)
	end
	-- Also limit to items we would want to stock for profit and reagents
	if item and ((hasTradeSkill and item.stackNum and item.stackNum > 0 and
	    item.stackSize and item.stackSize > 0 and item.minBuyout and 
	    item.reagents) or (not item.reagents and item.minBuyout)) then
	    self.list[#self.list + 1] = itemID
	end
    end
    total = #self.list

    if total > 0 then
	SetProgressText(1)
	self:DisableButtons()
	self:ScanList(function (itemID, _, _)

	    -- This item hasn't been removed yet
	    local left = #self.list - 1

	    if left == 0 then 
		self.progressFrame.Text:SetText("Slow scanning finished (" .. 
		    total ..  " total)")
		self:EnableButtons()
		Wait:Start(PROGRESS_CLOSE_DELAY, self.HideProgress, self)
		return
	    else
		SetProgressText(total - left + 1)
	    end
	end)
    end
end

-- Actually get the prices for a range of items on one page
function AH:GetPricesRange(first, last, singleItem)
    if self.stop or not self.open then return end

    for i = first, last do
	local _, _, total, _, _, _, _, _, _, buyout, _, _, _, _, _, _, itemID = 
	    GetAuctionItemInfo("list", i)
	if itemID and total and buyout and total > 0 then
	    if buyout > 0 then
		local item = _G.Crafty_Items[itemID]
		if item and item.minBuyout then
		    local buyoutEach = math.ceil(buyout / total)
		    -- If buyout is so low it could be rounded down to 0
		    if buyoutEach == 0 then buyoutEach = 1 end
		    if not self.seen[itemID] or buyoutEach < item.minBuyout 
			then
			_G.Crafty_Items[itemID].minBuyout = buyoutEach
			self.seen[itemID] = true
		    end
		    if singleItem then
			break
		    end
		end
	    end
	else
	    -- itemID, total and buyout never appear to fail
	    --Data:Debug("AH:GetPricesRange", "Retrying GetAuctionItemInfo")
	    Wait:Start(SCAN_DELAY, self.GetPricesRange, self, first, last, 
		singleItem)
	    return
	end
    end
end

-- Break up fast scan item processing into batches
--   All 100k  - locked up :p
--   Batch 50  - small lock up then smooth
--   Batch 500 - some lock ups
--   Batch 250 - small lock ups
function AH:GetBatchPrices(first, num, total)
    if self.stop or not self.open then return end

    -- Check total as old items expire
    local _, newTotal = GetNumAuctionItems("list")
    -- Offset the scan
    local offset = newTotal - total
    if offset == 0 then
	-- NOOP
    else
	--Data:Debug("AH:GetBatchPrices", "Offset", offset)
	first = first + offset
    end

    local last = first + num - 1
    self.progressFrame.Text:SetText("Getting prices for items " .. first .. 
	" to " ..  last .. " (".. total .. " total)")
    self:GetPricesRange(first, last, false)

    local left = newTotal - last
    if left > 0 then
	Wait:Start(BATCH_DELAY, self.GetBatchPrices, self, last + 1,
	    left < BATCH_LIMIT and left or BATCH_LIMIT, newTotal)
	return
    else
	self:EnableButtons()
	wipe(self.scanning)
	self.progressFrame.Text:SetText("Fast scan finished (".. total .. 
	    " total)")
	Wait:Start(PROGRESS_CLOSE_DELAY, self.HideProgress, self)
	--Data:Debug("AH:GetBatchPrices", "Scanning done")

	-- Debug: Check for missed items, really none on AH?
	--[[
	wipe(self.list)
	local items = _G.Crafty_Items
	for itemID in pairs(items) do
	    local item = items[itemID]
	    if self.seen[itemID] or (Data.vendorIDs[itemID] or (item and 
		item.isBoP)) then
		-- NOOP
	    else
		self.list[#self.list + 1] = itemID
	    end
	end
	if #self.list > 0 then
	    self:ScanList(function(itemID, seller, buyoutEach)

		if seller == nil and buyoutEach == nil then
		    -- None on AH, as expected
		else
		    Data:Debug("AH:GetBatchPrices", "Missed item", itemID, 
			"seller", seller, "buyoutEach", buyoutEach)
		end
	    end)
	end
	--]]
    end
end

-- AH List Update
function AH:OnListUpdate()
    if not self.open then return end

    self.scanning.query = nil
    if self.stop then
	self.stop = false
	return
    end
    local batch, total = GetNumAuctionItems("list")

    -- getAll scan
    if self.scanning.scan == "getAll" then
	self:GetBatchPrices(1, batch < BATCH_LIMIT and batch or BATCH_LIMIT, 
	    total)
    -- Item list scan
    elseif self.scanning.scan == "list" then
	self:GetListPrices(batch, total)
    -- Update prices after a search
    elseif self.scanning.scan == "search" then
	self:GetPricesRange(1, batch < BATCH_LIMIT and batch or BATCH_LIMIT, 
	    true)
    end
end

hooksecurefunc("AuctionFrameBrowse_Search", function ()
    local itemName = BrowseName:GetText()
    -- This works if the item is in the players inventory
    local _, itemLink = GetItemInfo(itemName)
    local itemID = Data:GetItemInfoFromLink(itemLink)

    -- Update prices, if possible
    local item = _G.Crafty_Items[itemID]
    if item and item.minBuyout then
	--[[
	Data:Debug("AuctionFrameBrowse_Search", "Updating price for", itemName)
	--]]
	AH.stop = false
	wipe(AH.seen)
	AH.scanning.scan = "search"
	AH.scanning.query = true
    end
end)

-- Stop buttons being enabled during getAll scan
-- BrowseSearchButton_OnUpdate doesn't appear to be used!
--[[
hooksecurefunc("BrowseSearchButton_OnUpdate", function ()
    if AH.scanning.scan == "getAll" then
	AH:DisableButtons()
    end
end)
--]]
